@@ -23,6 +23,8 @@
  * USA
  */
 
+#include "common.h"
+
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -30,6 +32,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <unistd.h>
+#include <errno.h>
 
 #include "buffer.h"
 #include "md5.h"
@@ -361,6 +365,22 @@ cherokee_buffer_read_file (cherokee_buffer_t *buf, char *filename)
 }
 
 
+ret_t 
+cherokee_buffer_multiply (cherokee_buffer_t *buf, int num)
+{
+	int i, initial_size;
+
+	initial_size = buf->len;
+	cherokee_buffer_ensure_size (buf, buf->len * num);
+
+	for (i=0; i<num; i++) {
+		cherokee_buffer_add (buf, buf->buf, initial_size);
+	}
+
+	return ret_ok;
+}
+
+
 ret_t
 cherokee_buffer_print_debug (cherokee_buffer_t *buf, int len)
 {
