@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2003 Alvaro Lopez Ortega
+ * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,11 +23,13 @@
  * USA
  */
 
-#include <stdarg.h>
+#include "common.h"
+
 #include <stdlib.h>
+#include <stdarg.h>
+#include <syslog.h>
 #include <string.h>
 
-#include "common.h"
 #include "log.h"
 
 /* Global mini-buffer for quickly use
@@ -54,18 +56,14 @@ void
 PRINT_ERROR (const char *format, ...)
 {
 	va_list arg_list;
+	CHEROKEE_TEMP(tmp, 2048);
 	
 	va_start(arg_list, format);
-
-	/* Print in the stderr and in the log
-	 */
-	vfprintf(stderr, format, arg_list);
-
-#ifdef HAVE_VSYSLOG
-	vsyslog (LOG_ERR, format, arg_list);
-#endif /* HAVE_VSYSLOG */
-
+	vsnprintf (tmp, tmp_size, format, arg_list);
 	va_end(arg_list);
+
+	fprintf (stderr, "%s", tmp);
+	syslog (LOG_ERR, tmp);
 }
 
 
