@@ -31,6 +31,7 @@
 #include <time.h>
 
 #include <sys/types.h>
+#include <sys/uio.h>
 #include <sys/socket.h>
 
 #include <arpa/inet.h>
@@ -68,12 +69,15 @@ cherokee_connection_new  (cherokee_connection_t **cnt)
 	n->log_at_end     = 0;
 	n->realm_ref      = NULL;
 	n->mmaped         = NULL;
+	n->thread         = NULL;
 
 	n->post        = NULL;
 	n->post_len    = 0;
 
 	cherokee_buffer_new (&n->buffer);
 	cherokee_buffer_new (&n->header_buffer);
+	cherokee_buffer_new (&n->incoming_header);
+
 	cherokee_buffer_new (&n->local_directory);
 	cherokee_buffer_new (&n->web_directory);
 	cherokee_buffer_new (&n->userdir);
@@ -120,6 +124,8 @@ cherokee_connection_free (cherokee_connection_t  *cnt)
 	
 	cherokee_buffer_free (cnt->buffer);
 	cherokee_buffer_free (cnt->header_buffer);
+	cherokee_buffer_free (cnt->incoming_header);
+
 	cherokee_buffer_free (cnt->local_directory);
 	cherokee_buffer_free (cnt->web_directory);
 	cherokee_buffer_free (cnt->userdir);
@@ -142,6 +148,8 @@ cherokee_connection_free (cherokee_connection_t  *cnt)
 ret_t
 cherokee_connection_clean (cherokee_connection_t *cnt)
 {	   
+	uint32_t header_len;
+
 	cnt->phase        = phase_reading_header;
 	cnt->error_code   = http_ok;
 	cnt->timeout      = -1;
@@ -174,10 +182,6 @@ cherokee_connection_clean (cherokee_connection_t *cnt)
 		cnt->post = NULL;
 	}
 	
-	cherokee_header_clean (cnt->header);
-
-	cherokee_buffer_clean (cnt->buffer);
-	cherokee_buffer_clean (cnt->header_buffer);
 	cherokee_buffer_clean (cnt->local_directory);
 	cherokee_buffer_clean (cnt->web_directory);
 	cherokee_buffer_clean (cnt->userdir);
@@ -192,7 +196,25 @@ cherokee_connection_clean (cherokee_connection_t *cnt)
 		cherokee_table_free2 (cnt->arguments, free);
 		cnt->arguments = NULL;
 	}
+
+	/* Drop out the last incoming header
+	 */
+	cherokee_header_get_length (cnt->header, &header_len);
 	
+	cherokee_header_clean (cnt->header);
+	cherokee_buffer_clean (cnt->buffer);
+	cherokee_buffer_clean (cnt->header_buffer);
+	cherokee_buffer_move_to_begin (cnt->incoming_header, header_len);
+
+	/* If the connection has incoming headers to be processed,
+	 * then increment the pending counter from the thread
+	 */	 
+	if (! cherokee_buffer_is_empty (cnt->incoming_header)) {
+		CONN_THREAD(cnt)->pending_conns_num++;
+	}
+
+	cnt->thread = NULL;
+
 	return ret_ok;
 }
 
@@ -252,9 +274,7 @@ send_buffer_unsafe (cherokee_connection_t *cnt)
 //	ret = write (SOCKET_FD(cnt->socket), cnt->buffer->buf, cnt->buffer->len);
 	ret = cherokee_socket_write (cnt->socket, cnt->buffer, &sent);
 
-	cherokee_buffer_make_empty (cnt->buffer);
-
-	if (ret <= ret_ok) {
+	if (ret < ret_ok) {
 		cnt->keepalive = 0;
 		return ret_error;
 	}
@@ -541,18 +561,18 @@ cherokee_connection_send_header_and_mmaped (cherokee_connection_t *cnt)
 
 
 ret_t
-cherokee_connection_recv (cherokee_connection_t *cnt, cherokee_buffer_t *buffer)
+cherokee_connection_recv (cherokee_connection_t *cnt, cherokee_buffer_t *buffer, uint32_t *len)
 {
 	ret_t   ret;
 	ssize_t readed;
 	
-//	ret = cherokee_buffer_read_from_fd (buffer, SOCKET_FD(cnt->socket), DEFAULT_RECV_SIZE, &readed);
 	ret = cherokee_socket_read (cnt->socket, buffer, DEFAULT_RECV_SIZE, &readed);
 	switch (ret) {
 	case ret_ok:
 		/* Add to the virtual host data sent counter
 		 */
 		CONN_VSRV(cnt)->data.rx += readed;
+		*len = readed;
 		return ret_ok;
 
 	case ret_eagain:
@@ -567,27 +587,26 @@ cherokee_connection_recv (cherokee_connection_t *cnt, cherokee_buffer_t *buffer)
 
 
 int
-cherokee_connection_eoh (cherokee_connection_t *cnt)
+cherokee_connection_eoh (cherokee_connection_t *cnt, cherokee_buffer_t *buffer, int tail_len)
 {
-	if (cherokee_buffer_is_empty(cnt->buffer))
-		return 0;
-	
-	if (cnt->buffer->len < 4)
-		return 0;
+	int   tail;
+	char *start;
 
-	/* In the most of requests it will be true
+	/* Too few information?
+	 * len("GET / HTTP/1.0" CRLF CRLF) = 18
 	 */
-	if ((cnt->buffer->buf[cnt->buffer->len-1] == '\n') &&
-	    (cnt->buffer->buf[cnt->buffer->len-2] == '\r') &&
-	    (cnt->buffer->buf[cnt->buffer->len-3] == '\n') &&
-	    (cnt->buffer->buf[cnt->buffer->len-4] == '\r'))
-	{
-		return 1;
+	if (buffer->len < 18) {
+		return 0;
 	}
 
+	/* Look for the starting point
+	 */
+	tail  = (tail_len < buffer->len) ? tail_len : buffer->len;
+	start = buffer->buf + (buffer->len - tail);
+
 	/* It could be a partial header, or maybe a POST request
 	 */
-	return (strstr(cnt->buffer->buf, CRLF CRLF) != NULL);
+	return (strstr(start, CRLF CRLF) != NULL);
 }
 
 
@@ -596,7 +615,7 @@ cherokee_connection_reading_check (cherokee_connection_t *cnt)
 {
 	/* Check for too long headers
 	 */
-	if (cnt->buffer->len > MAX_HEADER_LEN) {
+	if (cnt->incoming_header->len > MAX_HEADER_LEN) {
 		cnt->error_code = http_bad_request;
 		return ret_error;
 	}
@@ -693,7 +712,6 @@ cherokee_connection_step (cherokee_connection_t *cnt)
 
 	return_if_fail (cnt->handler != NULL, ret_error);
 
-
 	/* Need to 'read' from handler ?
 	 */
 	if (! cherokee_buffer_is_empty (cnt->buffer)) {
@@ -1013,7 +1031,7 @@ cherokee_connection_get_request (cherokee_connection_t *cnt)
 
 	/* Header parsing
 	 */
-	ret = cherokee_header_parse (cnt->header, cnt->buffer, header_type_request);
+	ret = cherokee_header_parse (cnt->header, cnt->incoming_header, header_type_request);
 	if (ret < ret_ok) goto error;
 
 	/* Maybe read the POST data
@@ -1031,7 +1049,10 @@ cherokee_connection_get_request (cherokee_connection_t *cnt)
 			return ret;
 		}
 
-		end = strstr(cnt->buffer->buf, CRLF CRLF);
+/* TODO !!!
+ * rewrite using header::_get_length
+ */
+		end = strstr(cnt->incoming_header->buf, CRLF CRLF);
 		if (end == NULL) {
 			cnt->error_code = http_bad_request;
 			return ret_error;
@@ -1039,9 +1060,9 @@ cherokee_connection_get_request (cherokee_connection_t *cnt)
 		
 		end += 4;
 
-		len = (cnt->buffer->buf + cnt->buffer->len) - end;
+		len = (cnt->incoming_header->buf + cnt->incoming_header->len) - end;
 		cherokee_buffer_add (cnt->post, end, len);
-		cherokee_buffer_drop_endding (cnt->buffer, len);
+		cherokee_buffer_drop_endding (cnt->incoming_header, len);
 	}
 
 	/* Copy the request
@@ -1120,19 +1141,19 @@ ret_t
 cherokee_connection_check_authentication (cherokee_connection_t *cnt, cherokee_handler_table_entry_t *plugin_entry)
 {
 	if (plugin_entry->validator_new_func != NULL) {
-		char *ptr;
 		ret_t ret;
+		char *ptr;
+		int   len;
 		cherokee_validator_t *validator = NULL;
 
-		/* Look for authentication in the headers
+		/* Look for authentication in the headers:
 		 * It's done on demand because the directory maybe don't have protection
 		 */
-		ptr = strstr (cnt->buffer->buf, "Authorization: ");
-		if (ptr == NULL) {
+		ret = cherokee_header_get_unknown (cnt->header, "Authorization", 13, &ptr, &len);
+		if (ret < ret_ok) {
 			goto unauthorized;
 		}
-		
-		ptr += 15;
+
 		ret = get_authorization (cnt, ptr);
 		if (ret != ret_ok) {
 			goto unauthorized;
