@@ -4,6 +4,7 @@
  *
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
+ *      Hiroshi Yamashita <piro7@SoftHome.net>
  *
  * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
  *
@@ -46,19 +47,23 @@
 #include "fdpoll.h"
 
 #ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
+# include <sys/time.h>
 #else 
-#include <time.h>
+# include <time.h>
 #endif
 
 #ifdef HAVE_SYS_RESOURCE_H
-#include <sys/resource.h>
-#else
-#include <resource.h>
+# include <sys/resource.h>
+#elif HAVE_RESOURCE_H
+# include <resource.h>
 #endif
 
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+# include <unistd.h>
+#endif
+
+#ifdef HAVE_ERROR_H
+# include <errno.h>
 #endif
 
 
@@ -157,7 +162,8 @@ fdpoll_epoll_new (cherokee_fdpoll_t **fdp)
 
 	   n->ep_fd = epoll_create (n->nfiles + 1);
 	   if (n->ep_fd < 0) {
-		   PRINT_ERROR ("Fatal Error: Couldn't get /dev/epoll descriptor: epoll_create(%d) -> %d\n", n->nfiles+1, n->ep_fd);
+		   PRINT_ERROR ("Fatal Error: Couldn't get epoll descriptor: epoll_create(%d): %s\n", 
+				n->nfiles+1, strerror(errno));
 		   exit (EXIT_EPOLL_CREATE);
 	   }
 
@@ -213,7 +219,8 @@ fdpoll_epoll_add (cherokee_fdpoll_t *fdp, int fd, int rw)
 
 	if (epoll_ctl (fdp->ep_fd, EPOLL_CTL_ADD, fd, &ev) < 0) 
 	{
-		PRINT_ERROR ("Error: epoll_ctl (EPOLL_CTL_ADD)\n");
+		PRINT_ERROR ("Error: epoll_ctl(%d, EPOLL_CTL_ADD, %d): %s\n", 
+			     fdp->ep_fd, fd, strerror(errno));
 		exit (EXIT_EPOLL_CTL_ADD);
 	}
 
@@ -259,7 +266,8 @@ fdpoll_epoll_del (cherokee_fdpoll_t *fdp, int fd)
 
 	if (epoll_ctl(fdp->ep_fd, EPOLL_CTL_DEL, fd, &ev) < 0)
 	{
-		PRINT_ERROR ("Error: epoll_ctl (EXIT_EPOLL_CTL_DEL)\n");
+		PRINT_ERROR ("Error: epoll_ctl(%d, EPOLL_CTL_DEL, %d): %s\n", 
+			     fdp->ep_fd, fd, strerror(errno));
 		exit (EXIT_EPOLL_CTL_DEL);
 	}
 
@@ -724,4 +732,198 @@ fdpoll_select_reset (cherokee_fdpoll_t *fdp, int fd)
 #endif /* USE_SELECT */
 
 
+/***********************************************************************/
+/* kqueue()                                                            */
+/*                                                                     */
+/* #include <sys/event.h>                                              */
+/*                                                                     */
+/* int kevent();                                                       */
+/*                                                                     */
+/***********************************************************************/
+#ifdef USE_KQUEUE
 
+#include <stdarg.h>
+static void _LOG(const char* msg, ...)
+{
+#ifndef SILENT
+        va_list l;
+
+        va_start(l, msg);
+        vfprintf(stderr, msg, l);
+        va_end(l);
+#endif
+}
+
+
+ret_t
+fdpoll_kqueue_new (cherokee_fdpoll_t **fdp)
+{
+          /* Get memory
+           */
+          CHEROKEE_NEW_STRUCT (n, fdpoll);
+
+          /* Look for max fd limit
+           */
+          n->nfiles = get_fd_limit();
+          n->nevents = 0;
+          n->nchanges = 0;
+                  
+       n->kq = kqueue();
+       
+          /* Get memory
+           */
+          n->eventlist = (struct kevent *) malloc (sizeof(struct kevent) * n->nfiles);
+          return_if_fail (n->eventlist, ret_nomem);
+          n->changelist = (struct kevent *) malloc (sizeof(struct kevent) * n->nfiles);
+          return_if_fail (n->changelist, ret_nomem);
+          
+          n->clidx = (int*) malloc (sizeof(int) * n->nfiles);
+          n->elidx = (int*) malloc (sizeof(int) * n->nfiles);
+          return_if_fail (n->clidx, ret_nomem);
+          return_if_fail (n->elidx, ret_nomem);
+
+       memset(n->changelist,0,sizeof(struct kevent) * n->nfiles);
+       memset(n->eventlist,0,sizeof(struct kevent) * n->nfiles);
+       memset(n->clidx,0,sizeof(int) * n->nfiles);
+       memset(n->elidx,0,sizeof(int) * n->nfiles);
+
+#ifdef HAVE_PTHREAD
+          pthread_rwlock_init(&n->rwlock, NULL); 
+#endif
+
+          /* Return it
+           */
+          *fdp = n;
+
+          return ret_ok;
+}
+
+ret_t
+fdpoll_kqueue_free (cherokee_fdpoll_t *fdp)
+{
+       free (fdp->eventlist);
+       free (fdp->changelist);
+       free (fdp->clidx);
+       free (fdp->elidx);
+       
+       free (fdp);
+       
+       return ret_ok;
+}
+
+ret_t
+fdpoll_kqueue_add (cherokee_fdpoll_t *fdp, int fd, int rw)
+{
+       LOCK_WRITER(fdp);
+
+       fdp->changelist[fdp->nchanges].ident = fd;
+       fdp->changelist[fdp->nchanges].flags = EV_ADD;
+       switch (rw)
+        {
+        case 0:  
+               fdp->changelist[fdp->nchanges].filter = EVFILT_READ; 
+               break;
+        case 1: 
+               fdp->changelist[fdp->nchanges].filter = EVFILT_WRITE; 
+               break;
+        }
+       fdp->clidx[fd] = fdp->nchanges;
+       fdp->nchanges++;
+
+//_LOG("adding fd %d rw %d\n",fd,rw);
+       UNLOCK(fdp);
+       
+       return ret_ok;
+}
+
+void inline
+fdpoll_kqueue_set_mode (cherokee_fdpoll_t *fdp, int fd, int rw)
+{
+//     LOCK_WRITER(fdp);
+
+//     fdp->changelist[fdp->clidx[fd]].filter = rw ? EVFILT_WRITE : EVFILT_READ;
+       fdpoll_kqueue_add(fdp,fd,rw);
+       //_LOG("set_mode fd %d rw %d\n",fd,rw);
+
+//     UNLOCK(fdp);
+}
+
+ret_t
+fdpoll_kqueue_del (cherokee_fdpoll_t *fdp, int fd)
+{
+       return ret_ok;
+/*
+       LOCK_WRITER(fdp);
+       fdp->changelist[fdp->nchanges].ident = fd;
+       fdp->changelist[fdp->nchanges].flags = EV_DELETE;
+       fdp->nchanges++;
+       //_LOG("del %d\n",fd);
+       UNLOCK(fdp);
+*/
+       return ret_ok;
+}
+
+inline int
+fdpoll_kqueue_watch (cherokee_fdpoll_t *fdp, int timeout_msecs)
+{
+       int ret,i;
+       struct timespec ts;
+
+//     LOCK_WRITER(fdp);
+
+       //_LOG("watch in %d check %d\n",timeout_msecs,fdp->nchanges);
+       ts.tv_sec = timeout_msecs / 1000L;
+       ts.tv_nsec = (timeout_msecs %1000L) * 1000000L;
+       ret = kevent (fdp->kq, fdp->changelist, fdp->nchanges, fdp->eventlist, fdp->nfiles, &ts);
+
+       fdp->nchanges=0;
+       if ( ret == -1) {
+               return -1;
+       }
+       for(i=0;i<ret;i++) {
+               fdp->elidx[fdp->eventlist[i].ident] = i;
+               //_LOG("ret %d watch out %d fd %d\n",ret,i,fdp->eventlist[i].ident);
+       }
+
+//     UNLOCK(fdp);
+       return ret;
+}
+
+int
+fdpoll_kqueue_check (cherokee_fdpoll_t *fdp, int fd, int rw)
+{
+       int ret,evidx;
+
+       evidx = fdp->elidx[fd];
+       if (fdp->eventlist[evidx].ident != fd) return 0;
+       if (fdp->eventlist[evidx].flags & EV_ERROR) return 0;
+
+       ret = 0;
+       switch (rw) {
+        case 0: 
+               ret = fdp->eventlist[evidx].filter == EVFILT_READ;
+               break;
+        case 1: 
+               ret = fdp->eventlist[evidx].filter == EVFILT_WRITE;
+               break;
+       }
+       //_LOG("checking %d is %d\n",fd,ret);
+       if (ret) fdpoll_kqueue_add(fdp,fd,rw);
+
+       return ret;
+}
+
+ret_t
+fdpoll_kqueue_reset (cherokee_fdpoll_t *fdp, int fd)
+{
+}
+
+ret_t 
+fdpoll_kqueue_get_method (cherokee_fdpoll_t *fdp, char **name)
+{
+       static char *method = "kqueue";
+
+       *name = method;
+       return ret_ok;
+}
+#endif /* USE_KQUEUE */
