@@ -4,6 +4,7 @@
  *
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
+ *      Hiroshi Yamashita <piro7@SoftHome.net>
  *
  * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
  *
@@ -32,6 +33,10 @@
 #if defined HAVE_EPOLL
 # include <sys/epoll.h>
 # define USE_EPOLL
+#elif defined HAVE_KQUEUE
+# include <sys/types.h>
+# include <sys/event.h>
+# define USE_KQUEUE
 #elif defined HAVE_SYS_POLL_H
 # include <sys/poll.h>
 # define USE_POLL
@@ -88,7 +93,17 @@ typedef struct {
 	int            npollfds;
 	int           *fdidx;
 #endif
-	
+
+#ifdef USE_KQUEUE
+        int  kq;
+        struct kevent *eventlist;
+        struct kevent *changelist;
+        int  nevents;
+        int  nchanges;
+        int  *clidx;
+        int  *elidx;
+#endif
+
 #ifdef USE_SELECT
 	int    *fd_rw;
 
@@ -134,6 +149,18 @@ typedef struct {
 # define cherokee_fdpoll_get_method fdpoll_poll_get_method
 #endif
 
+#ifdef USE_KQUEUE
+# define cherokee_fdpoll_new      fdpoll_kqueue_new
+# define cherokee_fdpoll_free     fdpoll_kqueue_free
+# define cherokee_fdpoll_add      fdpoll_kqueue_add
+# define cherokee_fdpoll_del      fdpoll_kqueue_del
+# define cherokee_fdpoll_set_mode fdpoll_kqueue_set_mode
+# define cherokee_fdpoll_watch    fdpoll_kqueue_watch
+# define cherokee_fdpoll_check    fdpoll_kqueue_check
+# define cherokee_fdpoll_reset    fdpoll_kqueue_reset
+# define cherokee_fdpoll_get_method fdpoll_kqueue_get_method
+#endif
+
 #ifdef USE_SELECT
 # define cherokee_fdpoll_new        fdpoll_select_new
 # define cherokee_fdpoll_free       fdpoll_select_free
@@ -146,6 +173,7 @@ typedef struct {
 # define cherokee_fdpoll_get_method fdpoll_select_get_method
 #endif
 
+
 ret_t cherokee_fdpoll_new        (cherokee_fdpoll_t **fdp);
 ret_t cherokee_fdpoll_free       (cherokee_fdpoll_t  *fdp);
 ret_t cherokee_fdpoll_get_method (cherokee_fdpoll_t  *fdp, char **name);   
