@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2003 Alvaro Lopez Ortega
+ * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,13 +23,18 @@
  * USA
  */
 
-#ifndef __CHEROKEE2_CONNECTION_HANDLER_COMMON_H__
-#define __CHEROKEE2_CONNECTION_HANDLER_COMMON_H_
+#ifndef __CHEROKEE_CONNECTION_HANDLER_COMMON_H__
+#define __CHEROKEE_CONNECTION_HANDLER_COMMON_H_
 
 #include "common.h"
 #include "handler.h"
 
-ret_t cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt);
+/* Library init function
+ */
+void common_init ();
 
 
-#endif /* __CHEROKEE2_CONNECTION_HANDLER_COMMON_H_ */
+ret_t cherokee_handler_common_new (cherokee_handler_t **hdl, void *cnt, cherokee_table_t *properties);
+
+
+#endif /* __CHEROKEE_CONNECTION_HANDLER_COMMON_H_ */
