@@ -108,6 +108,7 @@ cherokee_header_new (cherokee_header_t **hdr)
 	 */
 	n->input_buffer     = NULL;
 	n->input_buffer_crc = 0;
+	n->input_header_len = 0;
 
 	*hdr = n;
 	return ret_ok;
@@ -362,10 +363,11 @@ cherokee_header_parse (cherokee_header_t *hdr, cherokee_buffer_t *buffer,  chero
 	char  *begin = buffer->buf;
 	char  *end   = NULL;
 	char  *points;
+	char  *header_end;
 
 	/* Check the buffer content
 	 */
-	if ((buffer->buf == NULL) || (buffer->len < 2)) {
+	if ((buffer->buf == NULL) || (buffer->len < 5)) {
 		return ret_error;
 	}
 
@@ -374,12 +376,13 @@ cherokee_header_parse (cherokee_header_t *hdr, cherokee_buffer_t *buffer,  chero
 #ifdef HEADER_INTERNAL_DEBUG
 	hdr->input_buffer_crc = cherokee_buffer_crc32 (buffer);
 #endif
-
-	/* Parse the firdt line. Something like this:
-	 * GET /icons/compressed.png HTTP/1.1\r\n
+	/* Parse the speacial first line
 	 */
 	switch (type) {
 	case header_type_request:
+		/* Parse request. Something like this:
+		 * GET /icons/compressed.png HTTP/1.1\r\n
+		 */
 		ret = parse_request_first_line (hdr, buffer, &begin);
 		if (ret < ret_ok) return ret;
 		break;
@@ -398,10 +401,18 @@ cherokee_header_parse (cherokee_header_t *hdr, cherokee_buffer_t *buffer,  chero
 		SHOULDNT_HAPPEN;
 	}
 
+	/* Look for the header endding
+	 */
+	header_end = strstr (buffer->buf, CRLF CRLF);
+	if (header_end == NULL) {
+		return ret_error;
+	}
+	header_end += 4;
+
 	/* Parse the rest of headers
 	 */
-	while (end = get_new_line (begin))
-	{
+	while ((end = get_new_line(begin)) && (end < header_end))
+	{	       
 		points = index (begin, ':');
 		if (points == NULL) {
 			goto next;
@@ -473,13 +484,23 @@ cherokee_header_parse (cherokee_header_t *hdr, cherokee_buffer_t *buffer,  chero
 
 	next:
 		while ((*end == '\r') || (*end == '\n')) end++;
+
 		begin = end;
+		hdr->input_header_len = (end - hdr->input_buffer->buf);
 	}
 	
 	return ret_ok;
 }
 
 
+ret_t 
+cherokee_header_get_length (cherokee_header_t *hdr, uint32_t *len)
+{
+	*len = hdr->input_header_len;
+	return ret_ok;
+}
+
+
 ret_t 
 cherokee_header_get_unknown (cherokee_header_t *hdr, char *name, int name_len, char **header, int *header_len)
 {
