@@ -99,6 +99,7 @@ typedef struct {
 	 */
 	cherokee_buffer_t *input_buffer;
 	crc_t              input_buffer_crc;
+	uint32_t           input_header_len;
 
 } cherokee_header_t;
 
@@ -115,8 +116,9 @@ ret_t cherokee_header_free  (cherokee_header_t  *hdr);
 ret_t cherokee_header_clean (cherokee_header_t  *hdr);
 
 ret_t cherokee_header_parse         (cherokee_header_t *hdr, cherokee_buffer_t *buffer, cherokee_type_header_t type);
-ret_t cherokee_header_copy_request  (cherokee_header_t *hdr, cherokee_buffer_t *request);
+ret_t cherokee_header_get_length    (cherokee_header_t *hdr, uint32_t *len);
 
+ret_t cherokee_header_copy_request  (cherokee_header_t *hdr, cherokee_buffer_t *request);
 ret_t cherokee_header_get_arguments (cherokee_header_t *hdr, cherokee_buffer_t *qstring, cherokee_table_t *arguments);
 
 ret_t cherokee_header_has_known     (cherokee_header_t *hdr, cherokee_common_header_t header);
