@@ -36,23 +36,25 @@ typedef enum {
 typedef enum {
 	http_get,
 	http_post,
+	http_put,
 	http_head,
 	http_options,
 	http_unknown
 } cherokee_http_method_t;
 
 typedef enum {
-	   http_ok                = 200,
-	   http_accepted          = 202,
-	   http_partial_content   = 206,
-	   http_moved_permanently = 301,
-	   http_moved_temporarily = 302,
-	   http_not_modified      = 304,
-	   http_bad_request       = 400,
-	   http_unauthorized      = 401,
-	   http_access_denied     = 403,
-	   http_not_found         = 404,
-	   http_internal_error    = 500
+	http_unset             = 0,
+	http_ok                = 200,
+	http_accepted          = 202,
+	http_partial_content   = 206,
+	http_moved_permanently = 301,
+	http_moved_temporarily = 302,
+	http_not_modified      = 304,
+	http_bad_request       = 400,
+	http_unauthorized      = 401,
+	http_access_denied     = 403,
+	http_not_found         = 404,
+	http_internal_error    = 500
 } cherokee_http_t;
 
 /*                                      0        1         2         3         4         5         6         7
