@@ -5,7 +5,7 @@
  * Authors:
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  *
- * Copyright (C) 2001-2003 Alvaro Lopez Ortega
+ * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,7 +23,14 @@
  * USA
  */
 
+
+#include "common.h"
+
+#include <stdio.h>
 #include <stdarg.h>
+#include <syslog.h>
+#include <stdlib.h>
+#include <string.h>
 
 #include "log.h"
 
@@ -48,6 +55,20 @@ cherokee_log_close (void)
 }
 
 
+#ifndef HAVE_VSYSLOG
+void 
+vsyslog (int facility_priority, char *format, va_list arglist)
+{
+        char *msg = NULL;
+        vasprintf(&msg, format, arglist);
+        if (!msg)
+                return;
+        syslog(facility_priority, "%s", msg);
+        free(msg);
+}
+#endif /* HAVE_VSYSLOG */
+
+
 void inline
 cherokee_log (int priority, const char *format, ...)
 {	
@@ -55,9 +76,7 @@ cherokee_log (int priority, const char *format, ...)
 		va_list l;
 
 		va_start(l, format);
-#ifdef HAVE_VSYSLOG
 		vsyslog(priority, format, l);
-#endif /* HAVE_VSYSLOG */
 		va_end(l);
 	}
 }
