@@ -11,7 +11,7 @@
  *      Alvaro Lopez Ortega <alvaro@alobbs.com>
  * 
  *
- * Copyright (C) 2001, 2002, 2003 Alvaro Lopez Ortega
+ * Copyright (C) 2001, 2002, 2003, 2004 Alvaro Lopez Ortega
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -168,7 +168,7 @@ cherokee_logger_ncsa_flush (cherokee_logger_ncsa_t *logger)
 	}
 
 	if (logger->accesslog_fd == NULL) {
-		syslog (LOG_INFO, LOGGER_BUFFER(logger)->buf);
+		syslog (LOG_INFO, "%s", LOGGER_BUFFER(logger)->buf);
 		return cherokee_buffer_make_empty (LOGGER_BUFFER(logger));
 	}
 
@@ -239,7 +239,7 @@ cherokee_logger_ncsa_write_error  (cherokee_logger_ncsa_t *logger, cherokee_conn
 		return (wrote > 0) ? ret_ok : ret_error;
 	}
 
-	syslog (LOG_ERR, tmp);
+	syslog (LOG_ERR, "%s", tmp);
 	return ret_ok;
 }
 
@@ -254,7 +254,7 @@ cherokee_logger_ncsa_write_string (cherokee_logger_ncsa_t *logger, const char *s
 		return (ret > 0) ? ret_ok : ret_error;
 	} 
 
-	syslog (LOG_INFO, string);
+	syslog (LOG_INFO, "%s", string);
 	return ret_ok;
 }
 
