@@ -28,6 +28,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <limits.h>
 
 #include "mmap.h"
 #include "list_ext.h"
@@ -50,9 +51,6 @@ typedef struct {
 	char  *mmap;
 
 	int    use_count;
-//	size_t size;
-//	int    available;
-//	int    times_used;
 } cherokee_mmap_entry_t ;
 
 #define MMAP_ENTRY(e) ((cherokee_mmap_entry_t *)(e))
@@ -151,7 +149,7 @@ maybe_clean_up_table (cherokee_mmap_t *cmmap)
 {
 	int i;
 	list_t *a, *b;
-	int                    worse_usage = INT32_MAX;
+	int                    worse_usage = INT_MAX;
 	cherokee_mmap_entry_t *worse;
 
 	if (cmmap->items < MAX_MMAP_TABLE_ITEMS) {
