@@ -260,10 +260,19 @@ set_server_socket_opts (int socket)
 	 * Takes an integer value (seconds), this can bound the maximum number of
 	 * attempts TCP will make to complete the connection. This option should 
 	 * not be used in code intended to be portable.
+	 *
+	 * Give clients 10s to send first data packet 
 	 */
-	on = 5;
+#if defined (TCP_DEFER_ACCEPT)
+	on = 10;
 	re = setsockopt (socket, SOL_SOCKET, TCP_DEFER_ACCEPT, &on, sizeof(on));
 	if (re != 0) return ret_error;
+#endif
+
+	/* TODO:
+	 * Maybe add support for the FreeBSD accept filters:
+	 * http://www.freebsd.org/cgi/man.cgi?query=accf_http
+	 */
 
 	return ret_ok;
 }
@@ -453,23 +462,6 @@ for_each_vserver_init_tls_func (const char *key, void *value)
 #endif
 
 
-static void
-add_socket_to_thread_fdpolls (cherokee_server_t *srv, int srv_socket)
-{
-	list_t *i;
-
-	/* Add the main thread
-	 */
-	cherokee_fdpoll_add (srv->main_thread->fdpoll, srv_socket, 0);
-
-	/* The rest
-	 */
-	list_for_each (i, &srv->thread_list) {
-		cherokee_fdpoll_add (THREAD(i)->fdpoll, srv_socket, 0);		
-	}
-}
-
-
 ret_t
 cherokee_server_init (cherokee_server_t *srv) 
 {   
@@ -507,13 +499,15 @@ cherokee_server_init (cherokee_server_t *srv)
 		ret = initialize_server_threads (srv);
 		if (ret != ret_ok) return ret;
 	}
-
-	/* Add the server socket to fdpolls
+	
+	/* Chroot
 	 */
-	add_socket_to_thread_fdpolls (srv, srv->socket);
-#ifdef HAVE_TLS
-	add_socket_to_thread_fdpolls (srv, srv->socket_tls);
-#endif
+	if (srv->chroot) {
+		srv->chrooted = (chroot (srv->chroot) == 0);
+		if (srv->chrooted == 0) {
+			PRINT_ERROR ("Cannot chroot() to '%s': %s\n", srv->chroot, strerror(errno));
+		}
+	} 
 
 	/* Change the user
 	 */
@@ -521,15 +515,6 @@ cherokee_server_init (cherokee_server_t *srv)
 	if (ret != ret_ok) {
 		return ret;
 	}
-	
-	/* Chroot
-	 */
-	if (srv->chroot) {
-		srv->chrooted = (chroot (srv->chroot) == 0);
-		if (srv->chrooted == 0) {
-			PRINT_ERROR ("Cannot chroot() to '%s'\n", srv->chroot);
-		}
-	} 
 
 	chdir ("/");
 
