@@ -114,8 +114,8 @@ typedef struct {
 	int   hideversion;
 	int   hideservername;
 
-	int   keepalive;
-	int   keepalive_max;
+	cherokee_boolean_t keepalive;
+	uint32_t           keepalive_max;
 
 	uid_t  user;
 	uid_t  user_orig;
