@@ -91,10 +91,11 @@ cherokee_thread_new  (cherokee_thread_t **thd, void *server, int real_thread)
 	
 	cherokee_fdpoll_new(&n->fdpoll);
 
-	n->exit            = 0;
-	n->active_list_num = 0;
-	n->server          = server;
-	n->real_thread     = real_thread;
+	n->exit              = 0;
+	n->active_list_num   = 0;
+	n->pending_conns_num = 0;
+	n->server            = server;
+	n->real_thread       = real_thread;
 
 	/* Return the object
 	 */
@@ -194,9 +195,11 @@ maybe_purge_closed_connection (cherokee_thread_t *thread, cherokee_connection_t
 ret_t 
 process_active_connections (cherokee_thread_t *thd)
 {
-	ret_t ret;
+	ret_t    ret;
+	uint32_t len;
 	list_t *i, *tmp;
-
+	cherokee_boolean_t process;
+	
 	cherokee_connection_t *conn = NULL;
 	cherokee_server_t     *srv  = thd->server;
 
@@ -213,24 +216,48 @@ process_active_connections (cherokee_thread_t *thd)
 			continue;
 		}
 
-		/* Inspect the file descriptor
+		/* Process the connection?
+		 * 1.- Are there a pipelined connection?
 		 */
-		ret = cherokee_fdpoll_check (thd->fdpoll, 
-					     SOCKET_FD(conn->socket), 
-					     SOCKET_STATUS(conn->socket));
+		process = false;
+
+		if ((conn->phase == phase_reading_header) &&
+		    (!cherokee_buffer_is_empty (conn->incoming_header))) 
+		{
+			fflush(stdout);
+			process = true;
+		}
+			
+		/* Process the connection?
+		 * 2.- Inspect the file descriptor	
+		 */
+		if (process == false) {
+			ret = cherokee_fdpoll_check (thd->fdpoll, 
+						     SOCKET_FD(conn->socket), 
+						     SOCKET_STATUS(conn->socket));
+			
+			switch (ret) {
+			case -1:
+				purge_closed_connection(thd, conn);
+				continue;
+			case 0:
+				continue;
+			}
+
+			process = true;
+		}
 		
-		switch (ret) {
-		case -1:
-			purge_closed_connection(thd, conn);
-			continue;
-		case 0:
+		/* Process the connection?
+		 * Finial.- 
+		 */
+		if (process == false) {
 			continue;
 		}
 
+
 		/* The connection has work, so..
 		 */
 		conn->timeout = srv->bogo_now + srv->timeout;
-
 		
 		/* Phases
 		 */
@@ -250,7 +277,7 @@ process_active_connections (cherokee_thread_t *thd)
 
 
 		case phase_read_post:
-			ret = cherokee_connection_recv (conn, conn->post);
+			ret = cherokee_connection_recv (conn, conn->post, &len);
 			
 			switch (ret) {
 			case ret_ok:
@@ -287,8 +314,20 @@ process_active_connections (cherokee_thread_t *thd)
 			break;
 
 
-		case phase_reading_header:
-			ret = cherokee_connection_recv (conn, conn->buffer);
+		case phase_reading_header: 
+
+			/* Maybe the buffer has a request (previous pipelined)
+			 */
+			if (! cherokee_buffer_is_empty (conn->incoming_header)) {
+				if (cherokee_connection_eoh (conn, conn->incoming_header, 
+							     conn->incoming_header->len)) {
+					goto phase_reading_header_EXIT;
+				}
+			}
+			       			
+			/* Read from the client
+			 */
+			ret = cherokee_connection_recv (conn, conn->incoming_header, &len);
 			switch (ret) {
 			case ret_ok:
 				break;
@@ -301,7 +340,7 @@ process_active_connections (cherokee_thread_t *thd)
 				SHOULDNT_HAPPEN;
 			}
 
-			/* Check security
+			/* Check security after read
 			 */
 			ret = cherokee_connection_reading_check (conn);
 			if (ret != ret_ok) {
@@ -312,13 +351,15 @@ process_active_connections (cherokee_thread_t *thd)
 
 			/* May it already has the full header
 			 */
-			if (! cherokee_connection_eoh (conn)) {
+			if (! cherokee_connection_eoh (conn, conn->incoming_header, len+4)) {
 				continue;
 			}
 
+		phase_reading_header_EXIT:
 			conn->phase = phase_processing_header;
 
 		case phase_processing_header:
+
 			/* Get the request
 			 */
 			ret = cherokee_connection_get_request (conn);
@@ -456,7 +497,7 @@ process_active_connections (cherokee_thread_t *thd)
 			}
 			
 			/* If it has mmaped content, go to next stage
-			 */
+			 */		     
 			if (conn->mmaped != NULL) {
 				goto phase_add_headers_EXIT;
 			}
@@ -656,17 +697,25 @@ error:
 ret_t 
 cherokee_thread_step_SINGLE_THREAD (cherokee_thread_t *thd)
 {
-	cherokee_server_t *srv = THREAD_SRV(thd);
 	ret_t ret;
-	int re;
+	int   re;
+	cherokee_server_t *srv           = THREAD_SRV(thd);
+	int                fdwatch_msecs = srv->fdwatch_msecs;
 
 	cherokee_fdpoll_reset (thd->fdpoll, srv->socket);
 
-	re = cherokee_fdpoll_watch (thd->fdpoll, srv->fdwatch_msecs);
+	/* If thread has pending connections, it should do a 
+	 * faster 'watch' as possible
+	 */
+	if (thd->pending_conns_num > 0) {
+		fdwatch_msecs          = 0;
+		thd->pending_conns_num = 0;
+	}
+	
+	re = cherokee_fdpoll_watch (thd->fdpoll, fdwatch_msecs);
 	if (re <= 0) return ret_ok;
 
-	re = __accept_from_server (thd, srv->socket, 0);		
-
+	re = __accept_from_server (thd, srv->socket, 0);
 	return process_active_connections (thd);
 }
 
@@ -674,17 +723,26 @@ cherokee_thread_step_SINGLE_THREAD (cherokee_thread_t *thd)
 ret_t 
 cherokee_thread_step_MULTI_THREAD (cherokee_thread_t *thd)
 {
-	int i = MAX_NEW_CONNECTIONS_PER_STEP; 
-	int re;
 	ret_t ret;
-	cherokee_server_t *srv = THREAD_SRV(thd);
+	int   re;
+	cherokee_server_t *srv           = THREAD_SRV(thd);
+	int                fdwatch_msecs = srv->fdwatch_msecs;
+
+	/* If thread has pending connections, it should do a 
+	 * faster 'watch' as possible
+	 */
+	if (thd->pending_conns_num > 0) {
+		fdwatch_msecs          = 0;
+		thd->pending_conns_num = 0;
+	}
+
 
 	if (thd->active_list_num == 0) {
 		CHEROKEE_MUTEX_LOCK (&THREAD_SRV(thd)->accept_mutex);
 
 		cherokee_fdpoll_add   (thd->fdpoll, srv->socket, 0);
 		cherokee_fdpoll_reset (thd->fdpoll, srv->socket);
-		cherokee_fdpoll_watch (thd->fdpoll, srv->fdwatch_msecs);
+		cherokee_fdpoll_watch (thd->fdpoll, fdwatch_msecs);
 
 		re = __accept_from_server (thd, srv->socket, 0);		
 
@@ -699,7 +757,7 @@ cherokee_thread_step_MULTI_THREAD (cherokee_thread_t *thd)
 		if (! unlocked) {
 			cherokee_fdpoll_add   (thd->fdpoll, srv->socket, 0);
 			cherokee_fdpoll_reset (thd->fdpoll, srv->socket);
-			cherokee_fdpoll_watch (thd->fdpoll, srv->fdwatch_msecs);
+			cherokee_fdpoll_watch (thd->fdpoll, fdwatch_msecs);
 
 			re = __accept_from_server (thd, srv->socket, 0);		
 
@@ -707,7 +765,7 @@ cherokee_thread_step_MULTI_THREAD (cherokee_thread_t *thd)
 
 			CHEROKEE_MUTEX_UNLOCK (&THREAD_SRV(thd)->accept_mutex);
 		} else {
-			cherokee_fdpoll_watch (thd->fdpoll, srv->fdwatch_msecs);
+			cherokee_fdpoll_watch (thd->fdpoll, fdwatch_msecs);
 		}
 	}
 
@@ -740,9 +798,10 @@ cherokee_thread_get_new_connection (cherokee_thread_t *thd, cherokee_connection_
 
 	/* Set the basic information to the connection
 	 */
-	CONN_SRV(new_connection)  = server;
-	CONN_VSRV(new_connection) = server->vserver_default;
-	
+	CONN_THREAD(new_connection) = thd;
+	CONN_SRV(new_connection)    = server;
+	CONN_VSRV(new_connection)   = server->vserver_default;
+
 	/* Set the keepalive from server
 	 */
 	new_connection->keepalive = server->keepalive_max;
