@@ -91,14 +91,24 @@ validate_crypt (cherokee_connection_t *conn, const char *crypted)
 {
 	ret_t ret;
 	char *tmp;
+	char *passwd = "";
 
-#ifdef HAVE_PTHREAD 
-	tmp = crypt_r (conn->passwd->buf, crypted);
+	if (! cherokee_buffer_is_empty (conn->passwd)) {
+		passwd = conn->passwd->buf;
+	}
+
+#ifdef HAVE_PTHREAD
+	{
+		struct crypt_data data;
+
+		memset (&data, 0, sizeof(data));
+		tmp = crypt_r (passwd, crypted, &data);
+	}
 #else
-	tmp = crypt (conn->passwd->buf, crypted);
+	tmp = crypt (passwd, crypted);
 #endif
 
-	ret = (strcmp(crypted, tmp)) ? ret_error : ret_ok;
+	ret = (strcmp(tmp, crypted) == 0) ? ret_ok : ret_error;
 	return ret;
 }
 
@@ -110,7 +120,7 @@ validate_md5_apache (cherokee_connection_t *conn, const char *crypted)
 	 */
 	char out_buf[6 + 9 + 24 + 2]; 
 
-	PRINT_ERROR ("Apache special MD5 are still not supported\n");
+	PRINT_ERROR ("Apache special MD5 is still not supported\n");
 	return ret_error;
 }
 
@@ -182,8 +192,13 @@ cherokee_validator_htpasswd_check (cherokee_validator_htpasswd_t *htpasswd, cher
 	char *cryp;
 	int   cryp_len;
 	ret_t ret;
+	ret_t ret_auth;
 	CHEROKEE_TEMP(line, 128);
 
+	if (cherokee_buffer_is_empty (conn->user)) {
+		return ret_error;
+	}
+
 	/* 1.- Check the login/passwd
 	 */	  
 	f = fopen (htpasswd->htpasswd_file_ref, "r");
@@ -214,33 +229,40 @@ cherokee_validator_htpasswd_check (cherokee_validator_htpasswd_t *htpasswd, cher
 		*cryp++ = '\0';
 		cryp_len = strlen(cryp);
 		
+
 		/* Is this the right user? 
 		 */
 		if (strcmp (conn->user->buf, line) != 0) {
 			continue;
 		}
-		
+
 		/* Check the crypted password
 		 */
-//		if ((strncmp (cryp, "$1$",    3) == 0)) {    
-		if (strncmp (cryp, "$apr1$", 6) == 0)  /* Apache MD5 variant*/
+		if (strncmp (cryp, "$apr1$", 6) == 0)  
 		{    
-			ret = validate_md5_apache (conn, cryp);
-		} else if (cryp_len == 32) 
+			/* Apache MD5 variant
+			 */
+			ret_auth = validate_md5_apache (conn, cryp);
+		} 
+		else if (cryp_len == 32) 
 		{   
-			ret = validate_md5_digest (conn, cryp);
-		} else 
-		{
-			ret = validate_crypt (conn, cryp);
+			ret_auth = validate_md5_digest (conn, cryp);
+		} 
+		else {
+			ret_auth = validate_crypt (conn, cryp);
 		}
 
-		if (ret == ret_ok) {
-			break;
-		}
+		break;
 	}
 
 	fclose(f);
 
+	/* Check the authentication returned value
+	 */
+	if (ret_auth < ret_ok) {
+		return ret_auth;
+	}
+
 
 	/* 2.- Security check:
 	 * Is the client trying to download the passwd file?
